!*  <<< Subfit >>> ----------------------------------------------------
!+
!----------------------------------------------------------------------
!======================================================================

!*LESQ1    -- make a least-squares fit to data with a polynomial curve
!+
SUBROUTINE LESQ1(X,Y,SIGMAY,Npts,Nterms,MODE,A,chisqr)
  dimension        X(1),Y(1),SIGMAY(1),A(1)

!
! LESQ1:
!        Make a least-squares fit to data with a polynomial curve
!             Y = A(1) + A(2)*X + A(3)*X**2 + A(3)*X**3 + . . .
!
! Usage:
!        CALL LESQ1  (X, Y, SIGMAY, Npts, Nterms, MODE, A, chisqr)
!        CALL LESQ1  (X, Y, SIGMAY, 4, 4, 0, A, chisqr)
!        cf. CALL LESQ1  (X, Y, 4, -3, W, C, VW, IRC) ...... SSL-II
!
! Arguments:
!        Input/
!           X       : Array of data points for independent variable
!           Y       : Array of data points for dependent variable
!           Npts    : Number of pairs of data points
!           Nterms  : Number of coefficients (degree of polynomial + 1)
!           MODE    : Determines method of weighting least-squares fit
!                     +1 (instrumental) WEIGHT(I) = 1./SIGMAY(I)**2
!                      0 (no weighting) WEIGHT(I) = 1.
!                     -1 (statistical)  WEIGHT(I) = 1./Y(I)
!        Output/
!           SIGMAY  : Array of standard deviations for Y data points
!           chisqr  : Reduced chi square for fit
!        Work/
!           A       : Array of coefficients of polynomial
!
! Subroutines and function subprograms required:
!        DETERM (ARRAY, Norder)
!             Evaluates the determinant of a symetrical
!             two-dimensional matrix of order norder
!
! Comments:
!        dimension statement valid for Nterms up to 10
!--
!   Bevington, pages 140-142.
!       modified by M. Hamabe (1991-01-09)
!       Modified by M. Hamabe (2007-07-06)
!
!------------------------------------------------------------------------
  real(8) :: SUMX(19), SUMY(10), XTERM, YTERM, ARRAY(10,10), CHISQ, DETERM
	
!       Accumulate weighted sums
	
  NMAX=2*Nterms-1
  
  SUMX(1:NMAX)=0.0
  SUMY(1:Nterms)=0.0
  CHISQ=0.
  do I=1,Npts
     XI=X(I)
     Yi=Y(I)
     if ( mode>0 ) then
        if ( Yi>0 ) then
           weight=1.0/(-Yi)
        else if ( Yi==0 ) then
           weight=1.0
        else
           weight=1.0/Yi
        end if
     else if ( mode==0 ) then
        weight=1.0
     else
        weight=1.0/sigmay(i)**2
     end if
     XTERM=WEIGHT
     do N=1,NMAX
        SUMX(N)=SUMX(N)+XTERM
        XTERM=XTERM*XI
     end do
     YTERM=WEIGHT*Yi
     do N=1,Nterms
        SUMY(N)=SUMY(N)+YTERM
        YTERM=YTERM*XI
     end do
     CHISQ=CHISQ+WEIGHT*Yi**2
  end do
  
!       Construct matrices and calculate coefficients
  
  do J=1,Nterms
     do K=1,Nterms
        N=J+K-1
        ARRAY(J,K)=SUMX(N)
     end do
  end do
  DELTA=DETERM (ARRAY,Nterms)
  if ( delta==0 ) then
     chisqr=0.0
     A(1:Nterms)=0.0
     return
  end if
  do L=1,Nterms
     do J=1,Nterms
        do K=1,Nterms
           N=J+K-1
           ARRAY(J,K)=SUMX(N)
        end do
        ARRAY(J,L)=SUMY(J)
     end do
     A(L)=DETERM (ARRAY,Nterms) /DELTA
  end do
  
!       Calculate chi square
  
  do J=1,Nterms
     CHISQ=CHISQ-2.*A(J)*SUMY(J)
     do K=1,Nterms
        N=J+K-1
        CHISQ=CHISQ+A(J)*A(K)*SUMX(N)
     end do
  end do
  FREE = Npts - Nterms
  chisqr=CHISQ/FREE
  
  return
end SUBROUTINE LESQ1

!======================================================================
!
!*DLESQ1   -- make a least-squares fit to data with a polynomial curve (real*8)
!+
SUBROUTINE DLESQ1 (X,Y,SIGMAY,Npts,Nterms,MODE,A,chisqr)
  IMPLICIT REAL*8 (A-H,O-Z)
  dimension X(1),Y(1),SIGMAY(1),A(1)
  
! DLESQ1:
!   Make a least-squares fit to data with a polynomial curve
!      y = a(1) + a(2)*x + a(3)*x**2 + a(3)*x**3 + . . .
!
! Usage:
!   CALL LESQ1  (x, y, sigmay, npts, nterms, mode, a, chisqr)
!   CALL LESQ1  (x, y, sigmay, 4, 4, 0, a, chisqr)
! cf. CALL LESQ1  (X, Y,     4, -3, W, C, VW, IRC) ...... SSL-II
! cf. CALL LESQ1  (X, Y, NDATA,  M, W, C, VW, IRC) ...... SSL-II
!
! Description of parameters
!   x	   - array of data points for independent variable
!   y	   - array of data points for dependent variable
!   sigmay - array of standard deviations for y data points
!   npts   - number of pairs of data points
!   nterms - number of coefficients (degree of polynomial + 1)
!   mode   - determines method of weighting least-squares fit
!	     +1 (instrumental) weight(i) = 1./sigmay(i)**2
!	      0 (no weighting) weight(i) = 1.
!	     -1 (statistical)  weight(i) = 1./y(i)
!   a	   - array of coefficients of polynomial
!   chisqr - reduced chi square for fit
!
! Subroutines and function subprograms required
!   DETERM (array, norder)
!      evaluates the determinant of a symetrical
!      two-dimensional matrix of order norder
!
! Comments
!   dimension statement valid for nterms up TO 10
!
!--
!   Bevington, pages 140-142.
!       modified by M. Hamabe (1991-01-09)
!       Modified by M. Hamabe (2007-07-06)

!-----------------------------------------------------------------------

  real(8) :: SUMX(19), SUMY(10), ARRAY(10,10)
	
!       Accumulate weighted sums
	
  NMAX=2*Nterms-1
  SUMX(1:NMAX)=0.0D0
  SUMY(1:Nterms)=0.0D0
  CHISQ=0.
  do I=1,Npts
     XI=X(I)
     Yi=Y(I)
     if ( mode>0 ) then
        if ( Yi>0 ) then
           WEIGHT=1.0D0/(-Yi)
        else if( Yi==0 ) then
           WEIGHT=1.0D0
        else
           WEIGHT=1.0D0/Yi
        end if
     else if ( mode==0 )then
        WEIGHT=1.0D0
     else
        WEIGHT=1.0D0/SIGMAY(I)**2
     end if
     XTERM=WEIGHT
     do N=1,NMAX
        SUMX(N)=SUMX(N)+XTERM
        XTERM=XTERM*XI
     end do
     YTERM=WEIGHT*Yi
     do N=1,Nterms
        SUMY(N)=SUMY(N)+YTERM
        YTERM=YTERM*XI
     end do
     CHISQ=CHISQ+WEIGHT*Yi**2
  end do
  
!       Construct matrices and calculate coefficients

  do J=1,Nterms
     do K=1,Nterms
        N=J+K-1
        ARRAY(J,K)=SUMX(N)
     end do
  end do
  DELTA=DETERM (ARRAY,Nterms)
  if ( delta==0 ) then
     chisqr=0.0D0
     do J=1,Nterms
        A(J)=0.0D0
     end do
     return
  end if
  do L=1,Nterms
     do J=1,Nterms
        do K=1,Nterms
           N=J+K-1
           ARRAY(J,K)=SUMX(N)
        end do
        ARRAY(J,L)=SUMY(J)
     end do
     A(L)=DETERM (ARRAY,Nterms) /DELTA
  end do
  
!       Calculate chi square

  do J=1,Nterms
     CHISQ=CHISQ-2.*A(J)*SUMY(J)
     do K=1,Nterms
        N=J+K-1
        CHISQ=CHISQ+A(J)*A(K)*SUMX(N)
     end do
  end do
  FREE = Npts - Nterms
  chisqr=CHISQ/FREE
  
  return

end SUBROUTINE DLESQ1

!======================================================================
!
!*DETERM   -- calculate the determinant of a square matrix
!+
REAL*8 FUNCTION DETERM (ARRAY,Norder)
  IMPLICIT REAL*8 (A-H,O-Z)
  integer  :: Norder
  real(8)  :: ARRAY(10,10)
  
! FUNCTION DETERM:
!   Calculate the determinant of a square matrix
!
! Source:
!   Bevington, page 294.
!
! Usage:
!   DET = DETERM (ARRAY, Norder)
!
! Arguments:
!   ARRAY  - MATRIX
!   Norder - ORDER OF DETERMINANT (DEGREE OF MATRIX)
!
!
! Comments:
!   This subprogram destroys the input matrix array
!   dimension statement valid for norder up to 10

!--
!       modified by M. Hmamabe (1991-01-09)
!       Modified by M. Hamabe (2007-07-6)

!-----------------------------------------------------------------------
	
  DETERM=1.0D0
  do K=1, Norder
	   
!       Interchange columns if diagonal element iz zero	   
     if ( array(k,k)==0.0 ) then
        do J=K, Norder
           if( array(k,j)/=0.0 ) go to 31
        end do
        DETERM=0.0D0
        return
31      do I=K,Norder
           SAVE = ARRAY(I,J)
           ARRAY(I,J) = ARRAY(I,K)
           ARRAY(I,K) = SAVE
        end do
        DETERM = -DETERM
     end if
!       Subtract row k from lower rows to get diagonal matrix
     DETERM = DETERM*ARRAY(K,K)
     IF ( K-Norder>0 ) then
        K1=K+1
        do I=K1,Norder
           do J=K1,Norder
              ARRAY(I,J)=ARRAY(I,J)-ARRAY(I,K)*ARRAY(K,J)/ARRAY(K,K)
           end do
        end do
     end if
  end do
  
  return

end FUNCTION DETERM
!======================================================================
!
!*PLYNM1   -- compute the value of a one-dimensional polynomial.
!+
SUBROUTINE PLYNM1 (X,COEF,IORD,VAL)
  real(8) :: X, coef(IORD+1), VAL
!
! PLYNM1:
!         COMPUTE THE VALUE OF A ONE-DIMENSIONAL POLYNOMIAL.
!
! Arguments:
!         INPUT/
!           X      : INDEPENDENT VALIABLES
!           COEF   : COEFFICIENT OF THE POLYNOMIAL
!           IORD   : ORDER OF THE POLYNOMIAL
!         OUTPUT/
!           VAL    : VALUE OF THE POLYNOMIAL
!
! Note:
!         DOUBLE PRECISION ROUTINE/ IMPLICIT REAL*8 (A-H,O-Z)
!
!--
!     AUTHOR: S.OKAMURA 1985-09-27
!     checked: M. Hamabe (1991-01-09)
!---------------------------------------------------------------------

  VAL=0.0D0
  do J=1,IORD+1
     VAL=VAL+coef(J)*X**(J-1)
  end do
  
  return

end SUBROUTINE PLYNM1

!======================================================================

!*PLYNM2   -- compute the value of a two-dimensional polynomial.
!+
SUBROUTINE PLYNM2 (X,Y,COEF,IORD,VAL)
  real(8) :: coef(21),X,Y,VAL
!
! PLYNM2:
!         COMPUTE THE VALUE OF A TWO-DIMENSIONAL POLYNOMIAL.
!
! Arguments:
!         INPUT/
!           X,Y    : INDEPENDENT VALIABLES
!           COEF   : COEFFICIENT OF THE POLYNOMIAL
!           IORD   : ORDER OF THE POLYNOMIAL (MAX=5)
!         OUTPUT/
!           VAL    : VALUE OF THE POLYNOMIAL
!
! Note:
!         DOUBLE PRECISION ROUTINE/ IMPLICIT REAL*8 (A-H,O-Z)
!         MAXIMUM ORDER OF POLYNOMIAL = 5
!
!--
!     AUTHOR: M.WATANABE 1981
!     UPDATE: 1985-09-21 S.OKAMURA
!     checked: M. Hamabe (1991-01-09)
! ---------------------------------------------------------------------
  
  if ( IORD < 6 ) then
     VAL = coef(1)
     
     if(IORD==0) return
     VAL = VAL + coef(2)*X + coef(3)*Y
     
     if(IORD==1) return
     VAL = VAL + coef(4)*X**2 + coef(5)*X*Y + coef(6)*Y**2
     
     if(IORD==2) return
     VAL = VAL + coef(7)*X**3 + coef(8)*X**2*Y + coef(9)*X*Y**2 + coef(10)*Y**3
     
     if(IORD==3) return
     VAL = VAL + coef(11)*X**4 + coef(12)*X**3*Y + coef(13)*X**2*Y**2 &
          &    + coef(14)*X*Y**3 + coef(15)*Y**4
     
     if(IORD==4) return
     VAL = VAL + coef(16)*X**5 + coef(17)*X**4*Y + coef(18)*X**3*Y**2 &
          &    + coef(19)*X**2*Y**3 + coef(20)*X*Y**4 + coef(21)*Y**5
  end if

  return

END SUBROUTINE PLYNM2

!======================================================================
!
!*PLFIT1   -- least squares fit of a one-dimensional polynomial.
!+
SUBROUTINE PLFIT1 (X,Y,Ndata,IORD,COEF,RES,RESM,AIC)
  real(8) :: X(Ndata),Y(Ndata),RES(Ndata),coef(IORD+1),AIC,RESM

! PLFIT1:
!         LEAST SQUARES FIT OF A ONE-DIMENSIONAL POLYNOMIAL
!
! Arguments:
!         INPUT/
!           X,Y    : ARRAYS OF DATA TO BE FITTED BY A POLYNOMIAL
!                      Y = coef(1) + coef(2)*X + coef(3)*X**2 + ....
!           Ndata  : NUMBER OF DATA IN THE ARRAYS
!           IORD   : ORDER OF THE POLYNOMIAL
!         OUTPUT/
!           COEF   : MOST PROBABLE VALUES OF THE COEFFICIENTS
!           RES    : ARRAY OF RESIDUALS
!           RESM   : STANDARD DEVIATION (RMS) OF RESIDUALS
!           AIC    : AN INFORMATION CRITERION BY AKAIKE
!
! Other subroutines called:
!         PLYNM1
!         DLESQ1 ... subetc.f
!
! Note:
!         DOUBLE PRECISION ROUTINE/ IMPLICIT REAL*8 (A-H,O-Z)
!
!--
!     AUTHOR: S.OKAMURA  1985-06-26
!     UPDATE: 1985-09-27 S.OKAMURA
!     UPDATE: 1985-11-30 S.ICHIKAWA
!     UPDATE: 1990-01-09 M. Hamabe
! ---------------------------------------------------------------------

  real(8) :: SIGMAY, CHISQR, SUM, YC

!     *** LEAST SQUARES FITTING USING A SSL II SUBROUTINE ***
  call DLESQ1 (X,Y,SIGMAY,Ndata,IORD+1,0,COEF,CHISQR)

!     *** COMPUTE RESIDUALS AND STATISTICS ***
  SUM=0.0D0
  do I=1,Ndata
     call PLYNM1 (X(I),COEF,IORD,YC)
     RES(I)=Y(I)-YC
     SUM=SUM+RES(I)*RES(I)
  end do
  RESM=SQRT(SUM/REAL(Ndata))
  AIC=Ndata*LOG10(SUM)+2.0D0*REAL(IORD+1)
  
  return
END SUBROUTINE PLFIT1

!======================================================================
!
!*PLANE    -- least squares fit of a two-dimensional polynomial
!+
SUBROUTINE PLANE(X,Y,Z,Ndata,IORD,COEF,SD)
  IMPLICIT   REAL*8(A-H,O-Z)
  real(4) :: X(Ndata),Y(Ndata),Z(Ndata),SD

! PLANE:
!         LEAST SQUARES FIT OF A TWO-DIMENSIONAL POLYNOMIAL
!
! Arguments:
!         INPUT/
!           X,Y,Z  : ARRAYS OF DATA TO BE FITTED
!                       Z = F(X,Y)
!           Ndata  : NUMBER OF DATA IN THE ARRAYS
!           IORD   : ORDER OF THE POLYNOMIAL (0-5)
!         OUTPUT/
!           COEF   : MOST PROBABLE VALUES OF THE COEFFICIENT (REAL*8)
!           SD     : STANDARD DEVIATION (RMS) OF RESIDUALS
!
! Other subroutines called:
!         (A) GUSJRD2
!         (A) PLYNM2
!
! Note:
!         MAX.   OF IORD  = 5
!         THIS IS A VERY-HIGH-SPEED ROUTINE FOR LARGE NUMBER OF DATA.
!
!--
!     AUTHOR: M.WATANABE ?
!     UPDATE: M.HAMABE   1991-??-?? (for SPARC)
! ---------------------------------------------------------------------

  real(8) :: A(30,30), C(21), coef(21)

  JC=(IORD*IORD+3*IORD+2)/2
  JCP=JC+1
  
  A(1:30,1:30)=0.0D0
  
  do I=1, Ndata
     C(1)=1.0D0
     C(2)=X(I)
     C(3)=Y(I)
     if( IORD>1 ) then
        C(4)=C(2)**2
        C(5)=C(2)*C(3)
        C(6)=C(3)**2
     end if
     if( IORD>2 ) then
        C(7)=C(2)*C(4)
        C(8)=C(2)*C(5)
        C(9)=C(5)*C(3)
        C(10)=C(3)*C(6)
     end if
     if( IORD>3 ) then
        C(11)=C(4)*C(4)
        C(12)=C(7)*C(3)
        C(13)=C(4)*C(6)
        C(14)=C(2)*C(10)
        C(15)=C(6)*C(6)
     end if
     if( IORD>4 ) then 
        C(16)=C(4)*C(7)
        C(17)=C(11)*C(3)
        C(18)=C(7)*C(6)
        C(19)=C(4)*C(10)
        C(20)=C(2)*C(15)
        C(21)=C(6)*C(10)
     end if
     do II=1,JC
        do JJ=1,JC
           A(II,JJ)=A(II,JJ)+C(II)*C(JJ)
        end do
        A(II,JCP)=A(II,JCP)+DBLE(Z(I))*C(II)
     end do
  end do
  
  call GUSJRD2(A,JC,.1D-3)
  do II=1,JC
     C(II)=A(II,JCP)
     coef(II)=C(II)
  end do
  
  TT=0.0D0
  NT=0
  do I=1,Ndata
     XX=X(I)
     YY=Y(I)
     call PLYNM2(XX,YY,C,IORD,VAL)
     TT=TT+(DBLE(Z(I))-VAL)**2
     NT=NT+1
  end do
  SD=SQRT(SNGL(TT/NT))
  
  return

END SUBROUTINE PLANE

!-----------------------------------------------------------------------
SUBROUTINE GUSJRD2(B,N,ER)
  IMPLICIT REAL*8(A-H,O-Z)
  DIMENSION B(30,30)
  
  N1=N+1
  do K=1,N
     BMAX=DABS(B(K,K))
     IMAX=K
     K1=K+1
     if( K1<=N )  then
        do I=K1,N
           if( DABS(B(I,K))>BMAX ) then
              BMAX=DABS(B(I,K))
              IMAX=I
           end if
        end do
     end if

     if( BMAX<ER ) then
        print *, '--- No Solution ---'
        return
     end if

     if( IMAX/=K ) then
        do J=1,N1
           BP=B(K,J)
           B(K,J)=B(IMAX,J)
           B(IMAX,J)=BP
        end do
     end if

     W=B(K,K)
     do J=K,N1
        B(K,J)=B(K,J)/W
     end do
     do I=1,N
        if( I/=K ) then
           WP=B(I,K)
           do J=K,N1
              B(I,J)=B(I,J)-WP*B(K,J)
           end do
        end if
     end do
  end do
  
  return

END SUBROUTINE GUSJRD2
   
