!*  <<< SUBSMTH >>> ---------------------------------------------------
!======================================================================
!*GAUS1    -- perform 1-dimensional smoothing by a Gaussian beam.
!+
SUBROUTINE GAUS1 (SP,WORK,iext,SIGMA,WEIGHT,SW)
  real :: SP(iext),WORK(-50:iext+50),WEIGHT(-50:50)
!
! GAUS1:
!         PERFORM 1-DIMENSIONAL SMOOTHING BY A GAUSSIAN BEAM.
!
! Arguments:
!         INPUT/
!           iext   : DIMENSION OF THE ARRAY SP
!           SIGMA  : SIGMA OF THE GAUSSIAN BEAM IN PIXEL UNIT
!                      (MAX. 25.0)
!         INPUT,OUTPUT/
!           SP     : 1-DIMENSIONAL ARRAY TO BE SMOOTHED
!                      ON return SP CONTAINS THE SMOOTHED DATA
!         OUTPUT/
!           WEIGHT : WEIGHT PATTERN OF THE GAUSSIAN BEAM
!           SW     : SUM OF WEIGHT
!         WORK AREA/
!           WORK   : DIMENSION OF WORK AREA SHOULD BE LARGER
!                      THAN THAT OF SP BY 101
!                    DIMENSION WORK(iext+101) .... MAIN PROGRAM
!
!--
!     AUTHOR: S.OKAMURA 1985-10-12
!     UPDATE: 1986-08-18 S.OKAMURA (INCREASE BUFFER 21 --> 101)
! ---------------------------------------------------------------------


!     *** COMPUTE WEIGHT PATTERN AND SUM OF WEIGHT ***
  IWD=50
  SW=0.0
  HH=1.0/(SIGMA*SIGMA*2)
  do I=-IWD,IWD
     RR=REAL(I)*REAL(I)
     WEIGHT(I)=EXP(-HH*RR)*5.0
     SW=SW+WEIGHT(I)
  end do
  
!     *** STORE DATA INTO WORK AREA ***

  WORK(-iwd:0)=SP(-iwd:0)
  WORK(1:iext)=SP(1:iext)
  WORK(iext+1:iext+IWD)=SP(iext)
  
!     *** PERFORM SMOOTHING ***
  do I=1,iext
     SP(I)=0.0
     do II=-IWD,IWD
        SP(I)=SP(I)+WEIGHT(II)*WORK(I+II)
     end do
     SP(I)=SP(I)/SW
  end do

  return

END SUBROUTINE GAUS1

!======================================================================

!*MARGIN   -- fill margin of work area for 2-dimensional filtering.
!+
SUBROUTINE MARGIN (G,WORK,iext,jext,VALUE)
  real :: G(iext,jext),WORK(iext+8,jext+8)
!
! MARGIN:
!         FILL MARGIN OF WORK AREA FOR 2-DIMENSIONAL FILTERING.
!
! Arguments:
!         INPUT/
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           VALUE  : VALUE TO BE FILLED IN THE MARGIN OF WORK ARRAY
!           G      : 2-DIMENSIONAL ARRAY TO BE SMOOTHED
!         OUTPUT/
!           WORK   : 2-DIMENSIONAL ARRAY OF THE SAME CONTENT AS G
!                      BUT MARGIN IS FILLED WITH VAL
!
! Notes:
!         DIMENSION OF WORK AREA SHOULD BE LARGER THAN THAT OF G
!           BY 8.
!           DIMENSION WORK(iext+8,jext+8) ... MAIN PROGRAM
!
!--
!     AUTHOR: S.OKAMURA 1985-10-15
!     UPDATE:
! ---------------------------------------------------------------------
!
!
  do J=1,4
     do I=1,iext+8
        WORK(I,J)=VALUE
     end do
  end do
  do J=5,jext+4
     do I=1,4
        WORK(I,J)=VALUE
     end do
     do I=5,iext+4
        WORK(I,J)=G(I-4,J-4)
     end do
     do I=iext+5,iext+8
        WORK(I,J)=VALUE
     end do
  end do
  do J=jext+5,jext+8
     do I=1,iext+8
        WORK(I,J)=VALUE
     end do
  end do

  return

END SUBROUTINE MARGIN

!======================================================================

!*GAUSSV   -- filtering by variable-width Gaussian beams (intensity).
!+
SUBROUTINE GAUSSV(G,WORK,iext,jext,VALUE,THRES)
  real :: G(iext,jext), WORK(iext+8,jext+8)

! GAUSSV:
!         FILTERING BY VARIABLE-WIDTH GAUSSIAN BEAMS.
!
! Argumnets:
!         INPUT/
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           VALUE  : VALUE TO BE FILLED IN THE MARGIN OF WORK ARRAY
!           THRES  : THRESHOLD VALUE OF SUM OF INTENSITY
!         INPUT,OUTPUT/
!           G      : 2-DIMENSIONAL ARRAY TO BE FILTERED
!                      ON return, G CONTAINS THE FILTERED MATRIX
!         WORK AREA/
!           WORK   :
!
! Other subroutines called:
!         (A) MARGIN
!
! Note:
!         NUMBER OF BEAM PATTERN = 5
!           GAUSSIAN BEAM  SIGMA = 0.0 0.5 1.0 1.5 2.0
!
!--
!     AUTHOR: S.ICHIKAWA 1985-11-06
!     UPDATE: S.OKAMURA 1986-03-29 (NON-ZERO VALUE FOR VAL)
!     UPDATE: S.OKAMURA 1986-06-19 (VALUE IN ARGUMENT LIST)
! ---------------------------------------------------------------------

  integer :: IFLT(-4:4,-4:4,0:4), IAPT(-4:4,-4:4,0:4), ISW(0:4)
  
  do K=1,4
     ISW(K)=0
     SG=FLOAT(K)*0.5
     HH=1.0/(SG*SG*2.0)
     do J=-4,4
        do I=-4,4
           RR=I*I+J*J
           IFLT(I,J,K)=EXP(-HH*RR)*5.0+0.5
           if ( IFLT(I,J,K)>0 ) then
              IAPT(I,J,K)=1
           else
              IAPT(I,J,K)=0
           end if
           ISW(K)=ISW(K)+IFLT(I,J,K)
        end do
     end do
   end do
   do J=-4,4
      do I=-4,4
         IFLT(I,J,0)=0
         IAPT(I,J,0)=0
      end do
   end do
   IFLT(0,0,0)=5
   ISW(0)=5
   IAPT(0,0,0)=1

!     *** FILE MARGIN BY VALUE ***
   call MARGIN(G,WORK,iext,jext,VALUE)

   do J=1,jext
      do I=1,iext
         do K=4,0,-1
            W=0.0
            do M=-4,4
               do L=-4,4
                  W=W+WORK(I+L+4,J+M+4)*IAPT(L,M,K)
               end do
            end do
            KK=K
            if (W<THRES) go to 110
         end do
110      continue
         G(I,J)=0.0
         do M=-4,4
            do L=-4,4
               G(I,J)=G(I,J)+WORK(I+L+4,J+M+4)*FLOAT(IFLT(L,M,KK))
            end do
         end do
         G(I,J)=G(I,J)/FLOAT(ISW(KK))
      end do
   end do

   return

 END SUBROUTINE GAUSSV

!======================================================================

!*CRTFLT   -- create a filter for 2-dimensional filtering.
!+
SUBROUTINE CRTFLT (IFLT,IWIDTH,ISUM,COMENT)
  character :: COMENT*40
  integer   :: IFLT(-4:4,-4:4)
!
! CRTFLT:
!         CREATE A FILTER FOR 2-DIMENSIONAL FILTERING.
!
! Arguments:
!         INPUT/
!         OUTPUT/
!           IFLT   : 2-DIMENSIONAL FILTER FUNCTION
!           IWIDTH : HALF WIDTH OF THE FILTER (MAX=4)
!           ISUM   : SUM OF THE FILTER FUNCTION
!           COMENT : COMENT FOR HOUSE KEEPING BLOCK (CHARACTER*40)
!
! Other subroutines called:
!         (P) CRTFL1 (A) CRTGAU
!
!--
!     AUTHOR: S.OKAMURA  1985-10-15
!     UPDATE: S.ICHIKAWA 1985-11-06
!     UPDATE: S.OKAMURA  1985-11-30
!     UPDATE: M.Hamabe et al.   1991-03-25
! ---------------------------------------------------------------------
 
!       *** GAUSSIAN BEAM ***
  print *, ' '
  print *, 'GAUSSIAN BEAM FILTER'
  print *, '---- GAUSSIAN BEAM SMOOTHING; EXAMPLES OF WEIGHT PATTERN ----'
  print *, 'SIGMA  0.5       1.0           1.5                2.0        '
  print *, ' '
  print *, '                                                 1 1 1       '
  print *, '                              1 1 1          1 1 1 2 1 1 1   '
  print *, '                  1         1 2 2 2 1        1 2 3 3 3 2 1   '
  print *, '        1       2 3 2     1 2 3 4 3 2 1    1 1 3 4 4 4 3 1 1 '
  print *, '      1 5 1   1 3 5 3 1   1 2 4 5 4 2 1    1 2 3 4 5 4 3 2 1 '
  print *, '        1       2 3 2     1 2 3 4 3 2 1    1 1 3 4 4 4 3 1 1 '
  print *, '                  1         1 2 2 2 1        1 2 3 3 3 2 1   '
  print *, '                              1 1 1          1 1 1 2 1 1 1   '
  print *, '                                                 1 1 1       '
  print *, '                                                             '
  print *, '        9         29           73                 121        '
  print *, ' '
  print *, 'Enter sigma of the Gaussian beam (pixel unit; 0.5 - 2.0)'
  print '(''sigma ===> '',$)'
  read *, SG

  if ( SG<0.4 ) SG=0.4
  if ( SG>2.0 ) SG=2.0
!       *** COMPUTE WEIGHT PATTERN AND SUM OF WEIGHT ***
  call CRTGAU(SG,IFLT,IWIDTH,ISUM,IREC)

!     *** DISPLAY FILTER FUNCTION ***
  call CRTFL1 (IFLT,IWIDTH,ISUM)

  write(COMENT,'(''Gaussian beam; sigma='',f7.3)') sg

  return

END SUBROUTINE CRTFLT

! --- CRTFL1 ----------------------------------------------------------
SUBROUTINE CRTFL1 (IFLT,IWIDTH,ISUM)

!     *** DISPLAY FILTER FUNCTION ***

  character :: CW(-8:8,-4:4), C
  integer   :: IFLT(-4:4,-4:4)

!     *** CREATE FILTER FUNCTION CHARACTER MODE ***

  CW = ' '
  
  do J=-4,4
     do I=-4,4
        if ( IFLT(I,J)>0 ) then
           write(C,'(I1)') IFLT(I,J)
           CW(I*2,J)=C
        end if
     end do
  end do

!     *** DISPLAY FILTER FUNCTION ***
  print *, 'ADOPTED FILTER FUNCTION'
  do J=-IWIDTH,IWIDTH
     print '(20X,17A1)', (CW(I,J),I=-8,8)
  end do
  print '(10X,''SUM OF FILTER FUNCTION = '',I3)', ISUM
  print *

  return

END SUBROUTINE CRTFL1

!======================================================================

!*CRTGAU   -- create 2-dimensional Gaussian filter.
!+
SUBROUTINE CRTGAU(SG,IFLT,IWIDTH,ISUM,IREC)
  integer :: IFLT(-4:4,-4:4)

! CRTGAU:
!         CREATE 2-DIMENSIONAL GAUSSIAN FILTER.
!
! Arguments:
!         INPUT/
!           SG     : SIGMA OF GAUSSIAN BEAM
!         OUTPUT/
!           IFLT   : 2-DIMENSIONAL FILTER FUNCTION
!           IWIDTH : HALF WIDTH OF THE FILTER (MAX=4)
!           ISUM   : SUM OF THE FILTER FUNCTION
!           IREC   : return CODE ( 0 : NORMAL END )
!
!--
!     AUTHOR: S.ICHIKAWA 1985-12-14
! ---------------------------------------------------------------------

  if ( SG<=0.0 .OR. SG>2.0 ) then
     IREC=1
     return
  end if

!     *** COMPUTE WEIGHT PATTERN AND SUM OF WEIGHT ***
  ISUM=0
  HH=1.0/(SG*SG*2.0)
  do J=-4,4
     do I=-4,4
        RR=I*I+J*J
        IFLT(I,J)=EXP(-HH*RR)*5.0+0.5
        ISUM=ISUM+IFLT(I,J)
     end do
  end do

!     *** SET WIDTH OF WEIGHT PATTERN ***
  do I=1,4
     if ( IFLT(I,0)==0 ) go to 100
  end do
  I=5
  100 IWIDTH=I-1

  IREC=0

  return

END SUBROUTINE CRTGAU

!======================================================================
!*FLTR2D   -- filtering by a 2-dimensional digital filter.
!+
SUBROUTINE FLTR2D (G,WORK,iext,jext,VALUE,IFLT,IWIDTH,ISUM)
  real    :: G(iext,jext), WORK(iext+8,jext+8)
  integer :: IFLT(-4:4,-4:4)

! FLTR2D:
!         FILTERING BY A 2-DIMENSIONAL DIGITAL FILTER.
!
! Arguments:
!         INPUT/
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           VALUE  : VALUE TO BE FILLED IN THE MARGIN OF WORK ARRAY
!           IFLT   : FILTER FUNCTION
!           IWIDTH : HALF WIDTH OF THE FILTER FUNCTION (MAX=4)
!           ISUM   : SUM OF THE FILTER FUNCTION
!         INPUT,OUTPUT/
!           G      : 2-DIMENSIONAL ARRAY TO BE SMOOTHED
!                      ON return, G CONTAINS THE FILTERED MATRIX
!         WORK AREA/
!           WORK   :
!
! Other subroutines called:
!       (A) MARGIN
!
!--
!     AUTHOR: S.OKAMURA 1985-10-15
!     UPDATE: S.OKAMURA 1986-03-29 (NON-ZERO VALUE FOR VAL)
!     UPDATE: S.OKAMURA 1986-06-19 (VALUE IN ARGUMENT LIST)
! ---------------------------------------------------------------------
!
!     *** FILL MARGIN OF THE WORK ARRAY ***
  call MARGIN (G,WORK,iext,jext,VALUE)

!     *** PERFORM SMOOTHING ***
  do J=5,jext+4
     J1=J-4
     do I=5,iext+4
        I1=I-4
        G(I1,J1)=0.0
        do JJ=-IWIDTH,IWIDTH
           do II=-IWIDTH,IWIDTH
              G(I1,J1)=G(I1,J1)+REAL(IFLT(II,JJ))*WORK(I+II,J+JJ)
           end do
        end do
        G(I1,J1)=G(I1,J1)/REAL(ISUM)
     end do
  end do

  return

END SUBROUTINE FLTR2D

!======================================================================

!*BOXCAR   -- filtering by a boxcar filter.
!+
SUBROUTINE BOXCAR (G,WORK,iext,jext,VALUE,IWIDTH)
  real :: G(iext,jext), WORK(iext+8,jext+8)

! BOXCAR:
!         FILTERING BY A BOXCAR FILTER.
!
! Arguments:
!         INPUT/
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           VALUE  : VALUE TO BE FILLED IN THE MARGIN OF WORK ARRAY
!           IWIDTH : HALF WIDTH OF THE FILTER FUNCTION (MAX=4)
!         INPUT,OUTPUT/
!           G      : 2-DIMENSIONAL ARRAY TO BE SMOOTHED
!                      ON return, G CONTAINS THE FILTERED MATRIX
!         WORK AREA/
!           WORK   :
!
! Other subroutines called:
!         (A) MARGIN
!
! Note:
!         FAST ALGORITH IS ADOPTED AND FILTER FUNCTION IS NOT
!           USED EXPLICITLY.
!
!--
!     AUTHOR: S.OKAMURA 1985-10-15
!     UPDATE: S.OKAMURA 1986-03-29 (NON-ZERO VALUE FOR VAL)
!     UPDATE: S.OKAMURA 1986-06-19 (VALUE IN ARGUMENT LIST)
! ---------------------------------------------------------------------
  real :: DUM(51)

!     *** FILL MARGIN OF THE WORK ARRAY ***
  call MARGIN (G,WORK,iext,jext,VALUE)

!     *** PERFORM FILTERING (FAST ALGORITHM) ***
  IBOX=2*IWIDTH+1
  SIZE=REAL(IBOX*IBOX)
  do J=5,jext+4
     J1=J-4
     do I=5,iext+4
        I1=I-4
        G(I1,J1)=0.0
        if ( I==5 ) then
           DUM(1:IBOX)=0.0
           do II=-IWIDTH,IWIDTH
              K=II+IWIDTH+1
              do JJ=-IWIDTH,IWIDTH
                 DUM(K)=DUM(K)+WORK(I+II,J+JJ)
              end do
              G(I1,J1)=G(I1,J1)+DUM(K)
           end do
        else
           do K=2,IBOX
              DUM(K-1)=DUM(K)
           end do
           DUM(IBOX)=0.0
           do JJ=-IWIDTH,IWIDTH
              DUM(IBOX)=DUM(IBOX)+WORK(I+IWIDTH,J+JJ)
           end do
           do K=1,IBOX
              G(I1,J1)=G(I1,J1)+DUM(K)
           end do
        end if
        G(I1,J1)=G(I1,J1)/SIZE
     end do
  end do

  return

END SUBROUTINE BOXCAR

!======================================================================
!
!*MEDIAN   -- filtering by a median filter.
!+
SUBROUTINE MEDIAN (G,WORK,iext,jext,VALUE,IWIDTH)
  real :: G(iext,jext), WORK(iext+8,jext+8)

! MEDIAN:
!         FILTERING BY A MEDIAN FILTER.
!
! Arguments:
!         INPUT/
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           VALUE  : VALUE TO BE FILLED IN MARGIN OF WORK ARRAY
!           IWIDTH : HALF WIDTH OF THE WINDOW (MAX=4)
!         INPUT,OUTPUT/
!           G      : 2-DIMENSIONAL ARRAY TO BE FILTERED
!                      ON return, G CONTAINS THE FILTERED MATRIX
!         WORK AREA/
!           WORK   :
!
! Other subroutines called:
!        (A) MARGIN  (P) MEDIA1/ MEDIA2
!
!--
!     AUTHOR: S.OKAMURA 1985-10-15
!     UPDATE: S.OKAMURA 1986-03-29 (NON-ZERO VALUE FOR VAL)
!     UPDATE: S.OKAMURA 1986-06-19 (VALUE IN ARGUMENT LIST)
! ---------------------------------------------------------------------

  real    :: A(81), B(81)
  integer :: ID(81), IDB(81)

  IWD=2*IWIDTH+1
  NPIX=IWD*IWD
  IC=NPIX/2+1

  !     *** FILL MARGIN OF THE WORK ARRAY ***
  call MARGIN (G,WORK,iext,jext,VALUE)

  !     *** PERFORM FILTERING ***
  do J=5,jext+4
     J1=J-4
     do I=5,iext+4
        I1=I-4
        if ( I==5 ) then
           !           *** FOR THE FIRST PIXEL IN EACH ROW ***
           NP=0
           !           *** STORE DATA IN THE BUFFER ***
           do II=-IWIDTH,IWIDTH
              do JJ=-IWIDTH,IWIDTH
                 NP=NP+1
                 ID(NP)=NP
                 A(NP)=WORK(II+I,JJ+J)
              end do
           end do
           !      *** ARRANGE DATA IN THE BUFFER IN INCREASING ORDER ***
           call MEDIA1 (A,ID,NP)
           G(I1,J1)=A(IC)
        else
           !      *** FOR SUCCESSIVE PIXELS IN THE SAME ROW ***
           NP=0
           !      *** REJECT OLD DATA FROM THE BUFFER ***
           do K=1,NPIX
              ID(K)=ID(K)-IWD
              if ( ID(K)>0 ) then
                 NP=NP+1
                 IDB(NP)=ID(K)
                 B(NP)=A(K)
              end if
           end do
           ID(1:NP)=IDB(1:NP)
           A(1:NP)=B(1:NP)
           !     *** STORE NEW DATA IN THE RIGHT POSITION IN THE BUFFER ***
           do JJ=-IWIDTH,IWIDTH
              IDNUM=NPIX-IWIDTH+JJ
              DUM=WORK(I+IWIDTH,J+JJ)
              if ( DUM>=A(NP) ) then
                 NP=NP+1
                 A(NP)=DUM
                 ID(NP)=IDNUM
              else
                 do K=1,NP
                    if ( DUM<A(K) ) exit
                 end do
                 IGAP=K
                 do KK=NP,IGAP,-1
                    A(KK+1)=A(KK)
                    ID(KK+1)=ID(KK)
                 end do
                 A(IGAP)=DUM
                 ID(IGAP)=IDNUM
                 NP=NP+1
              end if
           end do
           G(I1,J1)=A(IC)
        end if
     end do
  end do

  return

END SUBROUTINE MEDIAN

! --- MEDIA1 ----------------------------------------------------------
SUBROUTINE MEDIA1 (RA,ID,NT)

!     *** SORT SUBROUTINE ***

  real    :: RA(NT)
  integer :: ID(NT)

  II=NT/2
1100 continue
  call MEDIA2 (RA,ID,II,NT)
  II=II-1
  if ( II>=2 ) go to 1100
  II=NT
1200 continue
  K=1
  call MEDIA2 (RA,ID,K,II)
  X=RA(1)
  RA(1)=RA(II)
  RA(II)=X
  IDUM=ID(1)
  ID(1)=ID(II)
  ID(II)=IDUM
  II=II-1
  if ( II>=2 ) go to 1200

  return

END SUBROUTINE MEDIA1

! --- MEDIA2 ----------------------------------------------------------
SUBROUTINE MEDIA2 (RA,ID,I,N)

!     *** SHIFT-UP SUBROUTINE ***

  real    :: RA(N)
  integer :: ID(N)

  ICOPY=ID(I)
  COPY=RA(I)
1000 continue
  J=2*I
  if ( J>N ) go to 1200
  if ( J>=N ) go to 1100
  if ( RA(J+1)<=RA(J) ) go to 1100
  J=J+1
1100 continue
  if ( RA(J)<=COPY ) go to 1200
  ID(I)=ID(J)
  RA(I)=RA(J)
  I=J
  go to 1000
1200 continue
  RA(I)=COPY
  ID(I)=ICOPY

  return

END SUBROUTINE MEDIA2

!======================================================================

!*VARGAU   -- filtering by a variable-width Gaussian filter (S.D.).
!+
SUBROUTINE VARGAU (G,WORK,iext,jext,VALUE,NLP,COMENT)
  character :: COMENT*40
  real      :: G(iext,jext), WORK(iext+8,jext+8)
  
! VARGAU:
!         FILTERING BY VARIABLE-WIDTH GAUSSIAN BEAMS.
!
! Arguments:
!         INPUT/
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           VALUE  : VALUE TO BE FILLED IN THE MARGIN OF WORK ARRAY
!           NLP    : LOGICAL UNIT NUMBER OF NLP
!         INPUT,OUTPUT/
!           G      : 2-DIMENSIONAL ARRAY TO BE FILTERED
!                      ON return, G CONTAINS THE FILTERED MATRIX
!           COMENT : COMMENT ON THE SMOOTHING PROCES (FOR HOUSE
!                      KEEPING BLOCK)
!         WORK AREA/
!           WORK   :
!
! Other subroutines called:
!         (A) MARGIN  (P) VARGA1/ VARGA2/ VARGA3
!
!--
!     AUTHOR: 1985-10-15 S.OKAMURA
!     UPDATE: 1985-11-25 S.OKAMURA
!     UPDATE: S.OKAMURA 1986-03-29 (NON-ZERO VALUE FOR VAL)
!     UPDATE: S.OKAMURA 1986-06-19 (VALUE IN ARGUMENT LIST)
!     UPDATE: T.AOKI    1987-08-07 (SDLIM INPUT ERROR)
! ---------------------------------------------------------------------
  character :: CW(-8:8,-4:4,4), CDUM, COM1*6, COM2*7
  real(8)   :: AMSD, SDSD
  real(4)   :: SD(4), SG(4)
  integer   :: ISUM(4), IWIDTH(4), NC(5), IFLT(-4:4,-4:4,4)

!     *** SET NUMBER OF BEAMS ***
  NBEAM=4

!     *** FILL MARGIN OF THE WORK ARRAY ***
  call MARGIN (G,WORK,iext,jext,VALUE)

!     *** CREATE STANDARD DEVIATION MATRIX IN G ***
  print *, 'HISTOGRAM OF S.D. IS BEING COMPUTED ...'
  IBOX=3
  call VARGA1 (G,WORK,iext,jext,IBOX)

!     *** COMPUTE MEAN AND S.D. OF THE HISTOGRAM ***
!           (FIVE CYCLES OF 2-SIGMA REJECTION)
  call VARGA2 (G,iext,jext,AMSD,SDSD)

!     *** SET BEAM PATTERN AND SMOOTHING MODE ***
  print *
  print *
  print *, '*------------- BEAM PATTERN ------------*'
  print *, '| BEAM WIDTH --> SG1   SG2   SG3   SG4  |'
  print *, '|                   (PIXEL UNIT)        |'
  print *, '|   1 : WIDE     2.0   1.5   1.0   0.5  |'
  print *, '|  <2>: NARROW   1.5   1.2   0.8   0.5  |'
  print *, '|   3 : MANUAL   ---   ---   ---   ---  |'
  print *, '*---------------------------------------*'
  print '(''  Enter beam pattern ( 1/<2>/3 ) ===> '',$)'
  read '(A1)', CDUM
  if ( CDUM/='1' .AND. CDUM/='3' ) CDUM='2'
  read (CDUM,'(I1)') KBEAM
  if ( KBEAM==1 ) then
     COM1='WIDE'
     SG(1)=2.0
     SG(2)=1.5
     SG(3)=1.0
     SG(4)=0.5
  else if ( KBEAM==2 ) then
     COM1='NARROW'
     SG(1)=1.5
     SG(2)=1.2
     SG(3)=0.8
     SG(4)=0.5
  else if ( KBEAM==3 ) then
     COM1='MANUAL'
     print '(''   Enter sg1, sg2, sg3, sg4(max=2.0) ===> '',$)'
     read *, SG(1),SG(2),SG(3),SG(4)
  end if
  print *
  print *, '*-------------- SMOOTHING MODE ---------------*'
  print *, '|                Range of S.D. to be smoothed |'
  print *, '|   1 : Deep      0  ---------> 300*sigma     |'
  print *, '|  <2>: Medium    0  ------> 50*sigma         |'
  print *, '|   3 : Shallow   0  ----> 25*sigma           |'
  print *, '|   4 : Manual    0  -------> SDlim*sigma     |'
  print *, '*---------------------------------------------*'
  print '(''  Enter smoothing mode ( 1/<2>/3/4 ) ===> '',$)'
  read '(A1)', CDUM
  if ( CDUM/='1' .AND. CDUM/='3' .AND. CDUM/='4' ) CDUM='2'
  read (CDUM,'(I1)') MODE
  if ( MODE==1 ) then
     COM2='DEEP'
     DUM=LOG10(300.0)
  else if ( MODE==2 ) then
     COM2='MEDIUM'
     DUM=LOG10(50.0)
  else if ( MODE==3 ) then
     COM2='SHALLOW'
     DUM=LOG10(25.0)
  else if ( MODE==4 ) then
     COM2='MANUAL'
     print '('' Enter SDlim  ===> '',$)'
     read *, SDLIM
     DUM=LOG10(SDLIM)
  end if

  do I=1,4
     SD(I)=AMSD+SDSD*10.0**(DUM*I/4.0)
  end do
  
  !     *** COMPUTE WEIGHT PATTERN AND SUM OF WEIGHT ***
  do K=1,NBEAM
     ISUM(K)=0
     HH=1.0/(SG(K)*SG(K)*2.0)
     do J=-4,4
        do I=-4,4
           RR=I*I+J*J
           IFLT(I,J,K)=EXP(-HH*RR)*5.0+0.5
           ISUM(K)=ISUM(K)+IFLT(I,J,K)
        end do
     end do
  end do

!     *** SET WIDTH OF WEIGHT PATTERN ***
  do K=1,NBEAM
     do I=1,4
        if ( IFLT(I,0,K)==0 ) go to 1100
     end do
     I=5
1100 IWIDTH(K)=I-1
  end do

!     *** WRITE PARAMETERS ON TERMINAL AND NLP ***
  print *
  call VARGA3 (IFLT,CW)
  write(NLP,'(///1H ,10X,"--- SMOOTHING BY VARIABLE-WIDTH GAUSSIAN BEAM:"/)')
  print *, ' ------------- VARIABLE-WIDTH GAUSSIAN-BEAM SMOOTHING (S.D.) ------------'
  print *
  print '(5X,4(6X,"BEAM",I1,6X)/"WIDTH:",4X,F5.1,3(12X,F5.1)/)', (K,K=1,NBEAM),(SG(I),I=1,NBEAM)
  write(NLP,'(1H ,15X,4(6X,"BEAM",I1,6X)/1H ,10X,"WIDTH:", &
       &      4X,F5.1,3(12X,F5.1)/)') (K,K=1,NBEAM),(SG(I),I=1,NBEAM)
  do J=-4,4
     print '(5X,4(17A1))', ((CW(I,J,K),I=-8,8),K=1,NBEAM)
     write(NLP,'(16X,4(17A1))') ((CW(I,J,K),I=-8,8),K=1,NBEAM)
  end do
  print *
  print '("SUM:",7X,3(I3,14X),I3)', (ISUM(I),I=1,NBEAM)
  write(NLP,'(/1H ,10X,"SUM:",7X,3(I3,14X),I3)') (ISUM(I),I=1,NBEAM)
  print '("THRES. S.D.:"," -----",F6.3,3(" ----------",F6.3))', (SD(I),I=1,NBEAM)
  write(NLP,'(1H ,10X,"THRES. S.D.:"," -----",F6.3,3(" ----------", F6.3))') &
       &     (SD(I),I=1,NBEAM)
  print *
  print *, 'DATA ARE BEING SMOOTHED ...'

  !     *** PERFORM SMOOTHING ***

  NC(1:NBEAM+1)=0
  
  do J=5,jext+4
     J1=J-4
     do I=5,iext+4
        I1=I-4
        DUM=0.0
        SD1=G(I1,J1)
        if ( SD1>SD(NBEAM) ) then
           G(I1,J1)=WORK(I,J)
           NC(NBEAM+1)=NC(NBEAM+1)+1
           cycle
        else if ( SD1>SD(3) ) then
           K=4
        else if ( SD1>SD(2) ) then
           K=3
        else if ( SD1>SD(1) ) then
           K=2
        else if ( SD1<=SD(2) ) then
           K=1
        end if
        NC(K)=NC(K)+1
        IBOX=IWIDTH(K)
        do JJ=-IBOX,IBOX
           do II=-IBOX,IBOX
              DUM=DUM+REAL(IFLT(II,JJ,K))*WORK(I+II,J+JJ)
           end do
        end do
        G(I1,J1)=DUM/REAL(ISUM(K))
     end do
  end do

  print *
  print '("PIXELS: ",I6,10X,2(I6,11X),I6,5X,I6)', (NC(I),I=1,NBEAM+1)
  write(NLP,'(1H ,10X,"PIXELS: ",I6,10X,3(I6,11X),I6)') (NC(I),I=1,NBEAM+1)
  
  !     *** WRITE COMMENT ***
  write(COMENT,'(''VarGau(S.D.);Beam='',A6,2X,''MODE='',A7)') COM1,COM2
  
  return
  
END SUBROUTINE VARGAU

! --- VARGA1 ----------------------------------------------------------
SUBROUTINE VARGA1 (G,WORK,iext,jext,IBOX)

!     *** CALCULATE STANDARD DEVIATION ***

  real(8)  :: BUFF1(9), BUFF2(9), DUM, AM, SD, SIZE
  real(4)  :: G(iext,jext), WORK(iext+8,jext+8)

  M2=2*IBOX+1
  SIZE=REAL(M2*M2)
  
  do J=5,jext+4
     J1=J-4
     do I=5,iext+4
        I1=I-4
        
        AM=0.0
        SD=0.0
        if ( I==5 ) then
           do K=1,M2
              BUFF1(K)=0.0
              BUFF2(K)=0.0
           end do
           do II=-IBOX,IBOX
              K=II+IBOX+1
              do JJ=-IBOX,IBOX
                 DUM=WORK(I+II,J+JJ)
                 BUFF1(K)=BUFF1(K)+DUM
                 BUFF2(K)=BUFF2(K)+DUM*DUM
              end do
              AM=AM+BUFF1(K)
              SD=SD+BUFF2(K)
           end do
           do K=2,M2
              BUFF1(K-1)=BUFF1(K)
              BUFF2(K-1)=BUFF2(K)
           end do
        else
           BUFF1(M2)=0.0
           BUFF2(M2)=0.0
           do JJ=-IBOX,IBOX
              DUM=WORK(I+IBOX,J+JJ)
              BUFF1(M2)=BUFF1(M2)+DUM
              BUFF2(M2)=BUFF2(M2)+DUM*DUM
           end do
           do K=1,M2
              AM=AM+BUFF1(K)
              SD=SD+BUFF2(K)
           end do
           do K=2,M2
              BUFF1(K-1)=BUFF1(K)
              BUFF2(K-1)=BUFF2(K)
           end do
        end if

        AM=AM/SIZE
        SD=SD/SIZE-AM*AM
        if ( SD<0.0 ) SD=0.0
        G(I1,J1)=SQRT(SD)

     end do
  end do

  return

END SUBROUTINE VARGA1

! --- VARGA2 ----------------------------------------------------------
SUBROUTINE VARGA2 (G,iext,jext,QM,QSD)
                                                                        
!     THIS SUBROUTINE COMPUTES MEAN AND S.D. OF THE MATRIX G.
!     FIVE CYCLES OF 2-SIGMA REJECTIONS ARE INCORPORATED.

  real(8) :: QM,QSD,DUM
  real(4) :: G(iext,jext)

!     *** COMPUTE STATISTICS OF INPUT DATA ***
  SIZE=REAL(iext*jext)
  QMAX=-1.0E30
  QMIN=1.0E30
  QM=0.0
  QSD=0.0
  do J=1,jext
     do I=1,iext
        DUM=G(I,J)
        QM=QM+DUM
        QSD=QSD+DUM*DUM
        if ( DUM>QMAX ) QMAX=DUM
        if ( DUM<QMIN ) QMIN=DUM
     end do
  end do
  QM=QM/SIZE
  QSD=QSD/SIZE-QM*QM
  if ( QSD < 0.0 ) QSD=0.0
  QSD=SQRT(QSD)
  NPIX=iext*jext
  print *
  print *, '------------ STATISTICS OF S.D. MATRIX ----------'
  print *, '         (FIVE CYCLES OF 2-SIGMA REJECTION)     '
  print *
  WRITE (*,'("INITIAL: MAX=",F7.3,2X,"MIN=",F7.3,2X,"MEAN=",F7.3,2X, &
       &     "S.D.=",F7.3,2X,"N=",I6)') QMAX,QMIN,QM,QSD,NPIX

!     *** 2-SIGMA REJECTION ***
  WRITE (*,*)
  do KK=1,5
     CRT=QSD*2.0
     AM=QM
     QMAX=-1.0E30
     QMIN=1.0E30
     QM=0.0
     QSD=0.0
     NPIX=0
     do J=1,jext
        do I=1,iext
           DUM=G(I,J)
           if ( ABS(DUM-AM)<=CRT ) then
              NPIX=NPIX+1
              if ( DUM>QMAX ) QMAX=DUM
              if ( DUM<QMIN ) QMIN=DUM
              QM=QM+DUM
              QSD=QSD+DUM*DUM
           end if
        end do
     end do
     QM=QM/REAL(NPIX)
     QSD=QSD/REAL(NPIX)-QM*QM
     if ( QSD < 0.0 ) QSD=0.0
     QSD=SQRT(QSD)
     WRITE (*,'("STEP=",I1,3X,"MAX=",F7.3,2X,"MIN=",F7.3,2X,"MEAN=",F7.3, &
          & 2X,"S.D.=",F7.3,2X,"N=",I6)') KK,QMAX,QMIN,QM,QSD,NPIX
  end do

  return

END SUBROUTINE VARGA2

! --- VARGA3 ----------------------------------------------------------
SUBROUTINE VARGA3 (IFLT,CW)

!     *** CREATE FILTER FUNCTION CHARACTER MODE ***

  character :: CW(-8:8,-4:4,4), C
  integer   :: IFLT(-4:4,-4:4,4)

  do K=1,4
     do J=-4,4
        do I=-8,8
           CW(I,J,K)=' '
        end do
     end do
     do J=-4,4
        do I=-4,4
           if (IFLT(I,J,K)>0) then
              write(C,'(I1)') IFLT(I,J,K)
              CW(I*2,J,K)=C
           end if
        end do
     end do
  end do
  
  return
  
END SUBROUTINE VARGA3

!======================================================================

!*MARVAL   -- compute mean value in margin of matrix for smoothing.
!+
SUBROUTINE MARVAL (G,iext,jext,VALUE,WORK,NBUFF,IRC)
  real  :: G(iext,jext),WORK(NBUFF)

! MARVAL:
!         COMPUTE MEAN VALUE IN MARGIN OF MATRIX FOR SMOOTHING.
!
! Arguments:
!         INPUT/
!           G      : INPUT MATRIX
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           WORK   : WORK AREA
!           NBUFF  : SIZE OF WORK AREA ; WORK(NBUFF)
!         OUTPUT/
!           VALUE  : MEAN VALUE IN THE MARGIN OF G
!                    (AFTER FIVE TIMES OF 2-SIGMA REJECTIONS)
!           IRC    : return CODE;
!                         0 = NORMAL END
!                         1 = BUFFER OVERFLOW (VALUE IS COMPUTED WITH
!                             FIRST NBUFF DATA POINT)
!                      1000 = MATRIX SIZE TOO SMALL
!                      1100 = TOO MANY DATA REJECTED
!
! Other subroutines called:
!         (A) STRJ1
!
!--
!     AUTHOR: 1986-06-20 S.OKAMURA
!     UPDATE:
! ---------------------------------------------------------------------

!     *** CHECK INPUT PARAMETERS ***
  if ( iext <= 10 .or. jext <= 10 ) then
     IRC=1000
     return
  end if

!     *** STORE MARGIN DATA IN WORK ARRAY ***
  IRC=0
  N=0
  do J=1,10
     do I=1,iext
        N=N+1
        if ( N > NBUFF ) go to 1000
        WORK(N)=G(I,J)
     end do
  end do
  do J=11,jext-10
     do I=1,10
        N=N+1
        if ( N > NBUFF ) go to 1000
        WORK(N)=G(I,J)
     end do
     do I=jext-9,jext
        N=N+1
        if ( N > NBUFF ) go to 1000
        WORK(N)=G(I,J)
     end do
  end do
  do J=jext-9,jext
     do I=1,iext
        N=N+1
        if ( N > NBUFF ) go to 1000
        WORK(N)=G(I,J)
     end do
  end do
  NDATA=N
  go to 1010
1000 continue
  NDATA=N-1
  IRC=1

!     *** FIVE-TIMES OF 2-SIGMA REJECTIONS ***
1010 continue
  FACT=2.0
  NCYCLE=5
  call STRJ1 (WORK,NDATA,FACT,NCYCLE,DMAX,DMIN,DMEAN,DSD,NREJ,KRET)
  if ( KRET >= 1000 ) then
     IRC=1100
     return
  end if
  VALUE=DMEAN

  return

END SUBROUTINE MARVAL

!======================================================================

!*MDFIL2   -- filtering by a median filter only for deviant pixels.
!+
SUBROUTINE MDFIL2 (G,WORK,iext,jext,VALUE,MTIME,ASIGMA,THRES,IWIDTH,IREC)
  real :: G(iext,jext),WORK(iext+8,jext+8)

! MDFIL2:
!         FILTERING BY A MEDIAN FILTER
!           FOR ONLY PIXELS    |G(I,J)-MEAN| >= T*SIGMA
!           ( MEAN,SIGMA ARE CALCULATED BY M-TIMES A*SIGMA REJECTIONS )
!
! Arguments:
!         INPUT/
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           VALUE  : VALUE TO BE FILLED IN MARGIN OF WORK ARRAY
!           MTIME  : NUMBER OF REJECTION PROCESSES
!           ASIGMA : LEVEL OF  REJECTION PROCESSES (ASIGMA*SIGMA)
!           THRES  : THRESHOLD LEVEL OF MEDIAN FILTERING (THRES*SIGMA)
!           IWIDTH : HALF WIDTH OF THE WINDOW (MAX=4)
!         OUTPUT/
!           IREC   : return CODE
!         INPUT,OUTPUT/
!           G      : 2-DIMENSIONAL ARRAY TO BE FILTERED
!                      ON return, G CONTAINS THE FILTERED MATRIX
!         WORK AREA/
!           WORK   :
!
! Other subroutines called:
!        (A) STATRJ/ MARGIN
!        (P) MDFL00
!
!--
!     AUTHOR: S.ICHIKAWA 1987-05-22
!     UPDATE: 1987-08-03  S.ICHIKAWA CORRECTION
! ---------------------------------------------------------------------

!     *** CHECK FILTER SIZE ***
  IREC=0
  if (IWIDTH>4) then
     IREC=10000
     return
  end if

!     *** CALCULATE SIGMA ***
  call STATRJ(G,iext,jext,1,iext,1,jext,ASIGMA,MTIME, &
       &      GMAX,GMIN,GMEAN,GSD,NREJ,KRET)
  if ( KRET/=0 ) then
     IREC=KRET
     return
  end if

!     *** FILL MARGIN OF THE WORK ARRAY ***
  call MARGIN (G,WORK,iext,jext,VALUE)
!
!     *** PERFORM FILTERING ***
  do J=5,jext+4
     JJ=J-4
     do I=5,iext+4
        II=I-4
        V=ABS( WORK(I,J)-GMEAN )/GSD
        if (V>=THRES) then
           call MDFL00(WORK,iext,jext,I,J,IWIDTH,F)
           G(II,JJ)=F
        else
           G(II,JJ)=WORK(I,J)
        end if
     end do
  end do

  return

END SUBROUTINE MDFIL2

!----------------------------------------------------------------------
SUBROUTINE MDFL00(WORK,iext,jext,II,JJ,KW,F)

!     *** CALCULATE MEDIAN ****

  real :: WORK(iext+8,jext+8), BUF(81)

  KDIM=(KW*2+1)**2
  KMED=(KDIM-1)/2+1
  
  K=0
  do J=JJ-KW,JJ+KW
     do I=II-KW,II+KW
        K=K+1
        BUF(K)=WORK(I,J)
     end do
  end do

  do K=1,KDIM
     NO1=0
     NO2=0
     do L=1,KDIM
        if (BUF(L)>=BUF(K)) NO1=NO1+1
        if (BUF(L)<=BUF(K)) NO2=NO2+1
     end do
     if ( NO1>=KMED .AND. NO2>=KMED ) then
        F=BUF(K)
        go to 1000
     end if
  end do
  F=WORK(II,JJ)

1000 continue

  return

END SUBROUTINE MDFL00

!======================================================================

!*MDFLIJ   -- filtering by an asymmetric median filter.
!+
SUBROUTINE MDFLIJ (G,WORK,iext,jext,VALUE,IW,JW)
  real   :: G(iext,jext),WORK(iext+8,jext+8)

! MDFLIJ:
!         FILTERING BY AN ASYMMETRIC MEDIAN FILTER
!
! Arguments:
!         INPUT/
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           VALUE  : VALUE TO BE FILLED IN MARGIN OF WORK ARRAY
!           IW     : HALF WIDTH OF THE WINDOW IN I-DIRECTION (MAX.4)
!           JW     : HALF WIDTH OF THE WINDOW IN J-DIRECTION (MAX.4)
!         INPUT,OUTPUT/
!           G      : 2-DIMENSIONAL ARRAY TO BE FILTERED
!                      ON return, G CONTAINS THE FILTERED MATRIX
!         WORK AREA/
!           WORK   :
!
! Other subroutines called:
!        (A) MARGIN  (P) MEDIA1/ MEDIA2
!
!--
!     AUTHOR: S.OKAMURA 1985-10-15
!     UPDATE: S.OKAMURA 1986-03-29 (NON-ZERO VALUE FOR VAL)
!     UPDATE: S.OKAMURA 1986-06-19 (VALUE IN ARGUMENT LIST)
!     UPDATE: S.OKAMURA 1987-08-03 (MODIFIED FROM MEDIAN)
! ---------------------------------------------------------------------
!
  real     :: A(81), B(81) 
  integer  :: ID(81), IDB(81)

  IWW=2*IW+1
  JWW=2*JW+1
  NPIX=IWW*JWW
  IC=NPIX/2+1
!
!     *** FILL MARGIN OF THE WORK ARRAY ***
  call MARGIN (G,WORK,iext,jext,VALUE)
  
  !     *** PERFORM FILTERING ***
  do J=5,jext+4
     J1=J-4
     do I=5,iext+4
        I1=I-4
        
        if ( I==5 ) then
           !      *** FOR THE FIRST PIXEL IN EACH ROW ***
           NP=0
           !      *** STORE DATA IN THE BUFFER ***
           do II=-IW,IW
              do JJ=-JW,JW
                 NP=NP+1
                 ID(NP)=NP
                 A(NP)=WORK(II+I,JJ+J)
              end do
           end do
           !      *** ARRANGE DATA IN THE BUFFER IN INCREASING ORDER ***
           call MEDIA1 (A,ID,NP)
           G(I1,J1)=A(IC)
        else
           !      *** FOR SUCCESSIVE PIXELS IN THE SAME ROW ***
           NP=0
           !      *** REJECT OLD DATA FROM THE BUFFER ***
           do K=1,NPIX
              ID(K)=ID(K)-JWW
              if ( ID(K) > 0 ) then
                 NP=NP+1
                 IDB(NP)=ID(K)
                 B(NP)=A(K)
              end if
           end do
           do K=1,NP
              ID(K)=IDB(K)
              A(K)=B(K)
           end do
           !     *** STORE NEW DATA IN THE RIGHT POSITION IN THE BUFFER ***
           do JJ=-JW,JW
              IDNUM=NPIX-JW+JJ
              DUM=WORK(I+IW,J+JJ)
              if ( DUM >= A(NP) ) then
                 NP=NP+1
                 A(NP)=DUM
                 ID(NP)=IDNUM
              else
                 do K=1,NP
                    if ( DUM < A(K) ) exit
                 end do
                 IGAP=K
                 do KK=NP,IGAP,-1
                    A(KK+1)=A(KK)
                    ID(KK+1)=ID(KK)
                 end do
                 A(IGAP)=DUM
                 ID(IGAP)=IDNUM
                 NP=NP+1
              end if
           end do
           G(I1,J1)=A(IC)
        end if
     end do
  end do

  return

END SUBROUTINE MDFLIJ

!======================================================================

!*MDFLTH   -- filtering by an median filter with threshold.
!+
SUBROUTINE MDFLTH (G,WORK,iext,jext,VALUE,IW,JW,FACT)
  real  :: G(iext,jext),WORK(iext+8,jext+8)

! MDFLTH:
!         FILTERING BY AN MEDIAN FILTER WITH THRESHOLD.
!
! Arguments:
!         INPUT/
!           iext   : I-DIMENSION OF THE MATRIX G
!           jext   : J-DIMENSION OF THE MATRIX G
!           VALUE  : VALUE TO BE FILLED IN MARGIN OF WORK ARRAY
!           IW     : HALF WIDTH OF THE WINDOW IN I-DIRECTION (MAX.4)
!           JW     : HALF WIDTH OF THE WINDOW IN J-DIRECTION (MAX.4)
!           FACT   : THRESHOLD; VALUE THAT DEVIATES MORE THAN
!                  :       FACT*SIGMA IS MEDIAN FILTERED
!         INPUT,OUTPUT/
!           G      : 2-DIMENSIONAL ARRAY TO BE FILTERED
!                      ON return, G CONTAINS THE FILTERED MATRIX
!         WORK AREA/
!           WORK   :
!
! Other subroutines called:
!        (A) MARGIN  (P) MEDIA1/ MEDIA2
!
!--
!     AUTHOR: S.OKAMURA 1989-03-19 (MODIFIED FROM MDFLIJ)
!     UPDATE: S.OKAMURA 1990-07-09 (ERROR)
! ---------------------------------------------------------------------
!
  real    :: A(81), B(81)
  integer :: ID(81), IDB(81)

  IWW=2*IW+1
  JWW=2*JW+1
  NPIX=IWW*JWW
  IC=NPIX/2+1

!     *** FILL MARGIN OF THE WORK ARRAY ***
  call MARGIN (G,WORK,iext,jext,VALUE)

!     *** PERFORM FILTERING ***
  do J=5,jext+4
     J1=J-4
     do I=5,iext+4
        I1=I-4
        
        if ( I == 5 ) then
           !           *** FOR THE FIRST PIXEL IN EACH ROW ***
           NP=0
           !           *** STORE DATA IN THE BUFFER ***
           do II=-IW,IW
              do JJ=-JW,JW
                 NP=NP+1
                 ID(NP)=NP
                 A(NP)=WORK(II+I,JJ+J)
              end do
           end do
           !           *** COMPUTE STATISTICS ***
           call STCS1 (A,NP,DMAX,DMIN,DMEAN,DSD)
           !           *** ARRANGE DATA IN THE BUFFER IN INCREASING ORDER ***
           call MEDIA1 (A,ID,NP)
           if ( ABS(G(I1,J1)-A(IC)) >= FACT*DSD) then
              G(I1,J1)=A(IC)
           end if
        else
           !           *** FOR SUCCESSIVE PIXELS IN THE SAME ROW ***
           NP=0
           !           *** REJECT OLD DATA FROM THE BUFFER ***
           do K=1,NPIX
              ID(K)=ID(K)-JWW
              if ( ID(K) > 0 ) then
                 NP=NP+1
                 IDB(NP)=ID(K)
                 B(NP)=A(K)
              end if
           end do
           do K=1,NP
              ID(K)=IDB(K)
              A(K)=B(K)
           end do
!           *** STORE NEW DATA IN THE RIGHT POSITION IN THE BUFFER ***
           do JJ=-JW,JW
              IDNUM=NPIX-JW+JJ
              DUM=WORK(I+IW,J+JJ)
              if ( DUM >= A(NP) ) then
                 NP=NP+1
                 A(NP)=DUM
                 ID(NP)=IDNUM
              else
                 do K=1,NP
                    if ( DUM<A(K) ) exit
                 end do
                 IGAP=K
                 do KK=NP,IGAP,-1
                    A(KK+1)=A(KK)
                    ID(KK+1)=ID(KK)
                 end do
                 A(IGAP)=DUM
                 ID(IGAP)=IDNUM
                 NP=NP+1
              end if
           end do
           !           *** COMPUTE STATISTICS ***
           call STCS1 (A,NP,DMAX,DMIN,DMEAN,DSD)
           !CCCCCCCCCC *** ARRANGE DATA IN THE BUFFER IN INCREASING ORDER ***
           !C 90/07 CC call MEDIA1 (A,ID,NP)
           if ( ABS(G(I1,J1)-A(IC)) >= FACT*DSD) then
              G(I1,J1)=A(IC)
           end if
        end if

     end do
  end do

  return

END SUBROUTINE MDFLTH
